@@ -3,6 +3,7 @@
 
 #include <Rcpp.h>
 #include <queue>
+#include <atomic>
 #include <functional>
 #include <memory>
 #include "timestamp.h"
@@ -38,8 +39,6 @@ public:
 
   virtual void invoke() const = 0;
 
-  void invoke_wrapped() const;
-
   virtual Rcpp::RObject rRepresentation() const = 0;
 
   Timestamp when;
@@ -56,15 +55,12 @@ public:
   StdFunctionCallback(Timestamp when, std::function<void (void)> func);
 
   void invoke() const {
-#ifdef RCPP_USING_UNWIND_PROTECT // See https://github.com/r-lib/later/issues/191
+    // See https://github.com/r-lib/later/issues/191 and https://github.com/r-lib/later/pull/241
     Rcpp::unwindProtect([this]() {
       BEGIN_RCPP
       func();
       END_RCPP
     });
-#else
-    func();
-#endif
   }
 
   Rcpp::RObject rRepresentation() const;
@@ -76,7 +72,7 @@ private:
 
 class RcppFunctionCallback : public Callback {
 public:
-  RcppFunctionCallback(Timestamp when, Rcpp::Function func);
+  RcppFunctionCallback(Timestamp when, const Rcpp::Function& func);
 
   void invoke() const {
     func();
@@ -115,7 +111,7 @@ private:
   // objects to be copied on the wrong thread, and even trigger an R GC event
   // on the wrong thread. https://github.com/r-lib/later/issues/39
   cbSet queue;
-  int fd_waits = 0;
+  std::atomic<int> fd_waits{};
   Mutex* mutex;
   ConditionVariable* condvar;
 
@@ -131,7 +127,7 @@ public:
 
   // Add a function to the registry, to be executed at `secs` seconds in
   // the future (i.e. relative to the current time).
-  uint64_t add(Rcpp::Function func, double secs);
+  uint64_t add(const Rcpp::Function& func, double secs);
 
   // Add a C function to the registry, to be executed at `secs` seconds in
   // the future (i.e. relative to the current time).
@@ -149,8 +145,8 @@ public:
   // Is anything ready to execute?
   bool due(const Timestamp& time = Timestamp(), bool recursive = true) const;
 
-  // Pop and return an ordered list of functions to execute now.
-  std::vector<Callback_sp> take(size_t max = -1, const Timestamp& time = Timestamp());
+  // Pop and return a function to execute now.
+  Callback_sp pop(const Timestamp& time = Timestamp());
 
   // Wait until the next available callback is ready to execute.
   bool wait(double timeoutSecs, bool recursive) const;
