@@ -20,10 +20,6 @@ bool at_top_level();
 bool execCallbacks(double timeoutSecs, bool runAll, int loop_id);
 bool idle(int loop);
 
-extern "C" uint64_t execLaterNative(void (*func)(void*), void* data, double secs);
-extern "C" uint64_t execLaterNative2(void (*func)(void*), void* data, double secs, int loop_id);
-extern "C" int apiVersion();
-
 void ensureInitialized();
 // Declare platform-specific functions that are implemented in later_posix.cpp
 // and later_win32.cpp.
